@@ -1114,6 +1114,12 @@ static GpStatus get_path_hrgn(GpPath *path, GpGraphics *graphics, HRGN *hrgn)
     GpStatus stat;
     INT save_state;
 
+    if (!path->pathdata.Count)  /* PathToRegion doesn't support empty paths */
+    {
+        *hrgn = CreateRectRgn( 0, 0, 0, 0 );
+        return *hrgn ? Ok : OutOfMemory;
+    }
+
     if (!graphics)
     {
         new_hdc = CreateCompatibleDC(0);
@@ -1396,11 +1402,7 @@ GpStatus WINGDIPAPI GdipIsVisibleRegionRect(GpRegion* region, REAL x, REAL y, RE
         return Ok;
     }
 
-    rect.left = ceilr(x);
-    rect.top = ceilr(y);
-    rect.right = ceilr(x + w);
-    rect.bottom = ceilr(y + h);
-
+    SetRect(&rect, ceilr(x), ceilr(y), ceilr(x + w), ceilr(y + h));
     *res = RectInRegion(hrgn, &rect);
 
     DeleteObject(hrgn);
