@@ -24,8 +24,6 @@
 #include <string.h>
 
 #define COBJMACROS
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 
 #include "winerror.h"
 #include "windef.h"
@@ -233,7 +231,7 @@ static HRESULT WINAPI ClassMoniker_BindToObject(IMoniker* iface,
     IClassActivator *pActivator;
     HRESULT hr;
 
-    TRACE("(%p,%p,%p,%p)\n", pbc, pmkToLeft, riid, ppvResult);
+    TRACE("(%p, %p, %s, %p)\n", pbc, pmkToLeft, debugstr_guid(riid), ppvResult);
 
     bindopts.cbStruct = sizeof(bindopts);
     IBindCtx_GetBindOptions(pbc, (BIND_OPTS *)&bindopts);
@@ -266,7 +264,7 @@ static HRESULT WINAPI ClassMoniker_BindToStorage(IMoniker* iface,
                                              REFIID riid,
                                              VOID** ppvResult)
 {
-    TRACE("(%p,%p,%p,%p)\n",pbc, pmkToLeft, riid, ppvResult);
+    TRACE("(%p, %p, %s, %p)\n", pbc, pmkToLeft, debugstr_guid(riid), ppvResult);
     return IMoniker_BindToObject(iface, pbc, pmkToLeft, riid, ppvResult);
 }
 
@@ -588,7 +586,7 @@ static HRESULT WINAPI ClassMonikerROTData_QueryInterface(IROTData *iface,REFIID
 
     ClassMoniker *This = impl_from_IROTData(iface);
 
-    TRACE("(%p,%p,%p)\n",iface,riid,ppvObject);
+    TRACE("(%p, %s, %p)\n", iface, debugstr_guid(riid), ppvObject);
 
     return IMoniker_QueryInterface(&This->IMoniker_iface, riid, ppvObject);
 }
