@@ -55,6 +55,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 typedef struct MPEGSplitterImpl
 {
     ParserImpl Parser;
+    IAMStreamSelect IAMStreamSelect_iface;
     LONGLONG EndOfFile;
     LONGLONG position;
     DWORD begin_offset;
@@ -64,11 +65,21 @@ typedef struct MPEGSplitterImpl
     BOOL seek;
 } MPEGSplitterImpl;
 
+static inline MPEGSplitterImpl *impl_from_IBaseFilter( IBaseFilter *iface )
+{
+    return CONTAINING_RECORD(iface, MPEGSplitterImpl, Parser.filter.IBaseFilter_iface);
+}
+
 static inline MPEGSplitterImpl *impl_from_IMediaSeeking( IMediaSeeking *iface )
 {
     return CONTAINING_RECORD(iface, MPEGSplitterImpl, Parser.sourceSeeking.IMediaSeeking_iface);
 }
 
+static inline MPEGSplitterImpl *impl_from_IAMStreamSelect( IAMStreamSelect *iface )
+{
+    return CONTAINING_RECORD(iface, MPEGSplitterImpl, IAMStreamSelect_iface);
+}
+
 static int MPEGSplitter_head_check(const BYTE *header)
 {
     /* If this is a possible start code, check for a system or video header */
@@ -476,7 +487,7 @@ static HRESULT MPEGSplitter_pre_connect(IPin *iface, IPin *pConnectPin, ALLOCATO
     HRESULT hr;
     LONGLONG pos = 0; /* in bytes */
     BYTE header[10];
-    int streamtype = 0;
+    int streamtype;
     LONGLONG total, avail;
     AM_MEDIA_TYPE amt;
     PIN_INFO piOutput;
@@ -514,12 +525,37 @@ static HRESULT MPEGSplitter_pre_connect(IPin *iface, IPin *pConnectPin, ALLOCATO
         hr = IAsyncReader_SyncRead(pPin->pReader, pos, 4, header);
         if (SUCCEEDED(hr))
             pos += 4;
-        TRACE("%x:%x:%x:%x\n", header[0], header[1], header[2], header[3]);
     } while (0);
 
-    while(SUCCEEDED(hr) && !(streamtype=MPEGSplitter_head_check(header)))
+    while(SUCCEEDED(hr))
     {
-        TRACE("%x:%x:%x:%x\n", header[0], header[1], header[2], header[3]);
+        TRACE("Testing header %x:%x:%x:%x\n", header[0], header[1], header[2], header[3]);
+
+        streamtype = MPEGSplitter_head_check(header);
+        if (streamtype == MPEG_AUDIO_HEADER)
+        {
+            LONGLONG length;
+            if (parse_header(header, &length, NULL) == S_OK)
+            {
+                BYTE next_header[4];
+                /* Ensure we have a valid header by seeking for the next frame, some bad
+                 * encoded ID3v2 may have an incorrect length and we end up finding bytes
+                 * like FF FE 00 28 which are nothing more than a Unicode BOM followed by
+                 * ')' character from inside a ID3v2 tag. Unfortunately that sequence
+                 * matches like a valid mpeg audio header.
+                 */
+                hr = IAsyncReader_SyncRead(pPin->pReader, pos + length - 4, 4, next_header);
+                if (FAILED(hr))
+                    break;
+                if (parse_header(next_header, &length, NULL) == S_OK)
+                    break;
+                TRACE("%x:%x:%x:%x is a fake audio header, looking for next...\n",
+                      header[0], header[1], header[2], header[3]);
+            }
+        }
+        else if (streamtype) /* Video or System stream */
+            break;
+
         /* No valid header yet; shift by a byte and check again */
         memmove(header, header+1, 3);
         hr = IAsyncReader_SyncRead(pPin->pReader, pos++, 1, header + 3);
@@ -553,8 +589,7 @@ static HRESULT MPEGSplitter_pre_connect(IPin *iface, IPin *pConnectPin, ALLOCATO
 
             if (FAILED(hr))
             {
-                if (amt.pbFormat)
-                    CoTaskMemFree(amt.pbFormat);
+                CoTaskMemFree(amt.pbFormat);
                 ERR("Could not create pin for MPEG audio stream (%x)\n", hr);
                 break;
             }
@@ -727,9 +762,36 @@ static HRESULT MPEGSplitter_first_request(LPVOID iface)
     return hr;
 }
 
+static HRESULT WINAPI MPEGSplitter_QueryInterface(IBaseFilter *iface, REFIID riid, void **ppv)
+{
+    MPEGSplitterImpl *This = impl_from_IBaseFilter(iface);
+    TRACE("(%s, %p)\n", qzdebugstr_guid(riid), ppv);
+
+    *ppv = NULL;
+
+    if ( IsEqualIID(riid, &IID_IUnknown)
+      || IsEqualIID(riid, &IID_IPersist)
+      || IsEqualIID(riid, &IID_IMediaFilter)
+      || IsEqualIID(riid, &IID_IBaseFilter) )
+        *ppv = iface;
+    else if ( IsEqualIID(riid, &IID_IAMStreamSelect) )
+        *ppv = &This->IAMStreamSelect_iface;
+
+    if (*ppv)
+    {
+        IBaseFilter_AddRef(iface);
+        return S_OK;
+    }
+
+    if (!IsEqualIID(riid, &IID_IPin) && !IsEqualIID(riid, &IID_IVideoWindow))
+        FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
+
+    return E_NOINTERFACE;
+}
+
 static const IBaseFilterVtbl MPEGSplitter_Vtbl =
 {
-    Parser_QueryInterface,
+    MPEGSplitter_QueryInterface,
     Parser_AddRef,
     Parser_Release,
     Parser_GetClassID,
@@ -746,6 +808,64 @@ static const IBaseFilterVtbl MPEGSplitter_Vtbl =
     Parser_QueryVendorInfo
 };
 
+static HRESULT WINAPI AMStreamSelect_QueryInterface(IAMStreamSelect *iface, REFIID riid, void **ppv)
+{
+    MPEGSplitterImpl *This = impl_from_IAMStreamSelect(iface);
+
+    return IBaseFilter_QueryInterface(&This->Parser.filter.IBaseFilter_iface, riid, ppv);
+}
+
+static ULONG WINAPI AMStreamSelect_AddRef(IAMStreamSelect *iface)
+{
+    MPEGSplitterImpl *This = impl_from_IAMStreamSelect(iface);
+
+    return IBaseFilter_AddRef(&This->Parser.filter.IBaseFilter_iface);
+}
+
+static ULONG WINAPI AMStreamSelect_Release(IAMStreamSelect *iface)
+{
+    MPEGSplitterImpl *This = impl_from_IAMStreamSelect(iface);
+
+    return IBaseFilter_Release(&This->Parser.filter.IBaseFilter_iface);
+}
+
+static HRESULT WINAPI AMStreamSelect_Count(IAMStreamSelect *iface, DWORD *streams)
+{
+    MPEGSplitterImpl *This = impl_from_IAMStreamSelect(iface);
+
+    FIXME("(%p/%p)->(%p) stub!\n", This, iface, streams);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI AMStreamSelect_Info(IAMStreamSelect *iface, LONG index, AM_MEDIA_TYPE **media_type, DWORD *flags, LCID *lcid, DWORD *group, WCHAR **name, IUnknown **object, IUnknown **unknown)
+{
+    MPEGSplitterImpl *This = impl_from_IAMStreamSelect(iface);
+
+    FIXME("(%p/%p)->(%d,%p,%p,%p,%p,%p,%p,%p) stub!\n", This, iface, index, media_type, flags, lcid, group, name, object, unknown);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI AMStreamSelect_Enable(IAMStreamSelect *iface, LONG index, DWORD flags)
+{
+    MPEGSplitterImpl *This = impl_from_IAMStreamSelect(iface);
+
+    FIXME("(%p/%p)->(%d,%x) stub!\n", This, iface, index, flags);
+
+    return E_NOTIMPL;
+}
+
+static const IAMStreamSelectVtbl AMStreamSelectVtbl =
+{
+    AMStreamSelect_QueryInterface,
+    AMStreamSelect_AddRef,
+    AMStreamSelect_Release,
+    AMStreamSelect_Count,
+    AMStreamSelect_Info,
+    AMStreamSelect_Enable
+};
+
 HRESULT MPEGSplitter_create(IUnknown * pUnkOuter, LPVOID * ppv)
 {
     MPEGSplitterImpl *This;
@@ -769,10 +889,11 @@ HRESULT MPEGSplitter_create(IUnknown * pUnkOuter, LPVOID * ppv)
         CoTaskMemFree(This);
         return hr;
     }
+    This->IAMStreamSelect_iface.lpVtbl = &AMStreamSelectVtbl;
     This->seek = TRUE;
 
     /* Note: This memory is managed by the parser filter once created */
-    *ppv = This;
+    *ppv = &This->Parser.filter.IBaseFilter_iface;
 
     return hr;
 }
