@@ -63,16 +63,18 @@ static BOOL ME_DITypesEqual(ME_DIType type, ME_DIType nTypeOrClass)
   }
 }
 
-/* Modifies run pointer to point to the next run, and modify the
- * paragraph pointer if moving into the next paragraph.
+/* Modifies run pointer to point to the next run.
+ * If all_para is FALSE constrain the search to the current para,
+ * otherwise modify the paragraph pointer if moving into the next paragraph.
  *
  * Returns TRUE if next run is found, otherwise returns FALSE. */
-BOOL ME_NextRun(ME_DisplayItem **para, ME_DisplayItem **run)
+BOOL ME_NextRun(ME_DisplayItem **para, ME_DisplayItem **run, BOOL all_para)
 {
   ME_DisplayItem *p = (*run)->next;
   while (p->type != diTextEnd)
   {
     if (p->type == diParagraph) {
+      if (!all_para) return FALSE;
       *para = p;
     } else if (p->type == diRun) {
       *run = p;
@@ -83,16 +85,18 @@ BOOL ME_NextRun(ME_DisplayItem **para, ME_DisplayItem **run)
   return FALSE;
 }
 
-/* Modifies run pointer to point to the previous run, and modify the
- * paragraph pointer if moving into the previous paragraph.
+/* Modifies run pointer to point to the previous run.
+ * If all_para is FALSE constrain the search to the current para,
+ * otherwise modify the paragraph pointer if moving into the previous paragraph.
  *
  * Returns TRUE if previous run is found, otherwise returns FALSE. */
-BOOL ME_PrevRun(ME_DisplayItem **para, ME_DisplayItem **run)
+BOOL ME_PrevRun(ME_DisplayItem **para, ME_DisplayItem **run, BOOL all_para)
 {
   ME_DisplayItem *p = (*run)->prev;
   while (p->type != diTextStart)
   {
     if (p->type == diParagraph) {
+      if (!all_para) return FALSE;
       if (p->member.para.prev_para->type == diParagraph)
         *para = p->member.para.prev_para;
     } else if (p->type == diRun) {
