@@ -261,7 +261,8 @@ static void draw_space( ME_Context *c, ME_Run *run, int x, int y,
 
     SetRect( &rect, x, ymin, x + run->nWidth, ymin + cy );
 
-    if (c->editor->bHideSelection) selected = FALSE;
+    if (c->editor->bHideSelection || (!c->editor->bHaveFocus &&
+                !(c->editor->styleFlags & ES_NOHIDESEL))) selected = FALSE;
     if (c->editor->bEmulateVersion10)
     {
         old_style_selected = selected;
@@ -355,7 +356,8 @@ static void ME_DrawTextWithStyle(ME_Context *c, ME_Run *run, int x, int y,
   HGDIOBJ hOldFont;
   int yOffset = 0;
   BOOL selected = (nSelFrom < run->len && nSelTo >= 0
-                   && nSelFrom < nSelTo && !c->editor->bHideSelection);
+                   && nSelFrom < nSelTo && !c->editor->bHideSelection &&
+                   (c->editor->bHaveFocus || (c->editor->styleFlags & ES_NOHIDESEL)));
   BOOL old_style_selected = FALSE;
   RECT sel_rect;
   HRGN clip = NULL, sel_rgn = NULL;
