@@ -31,6 +31,8 @@
 #define EXPECT_HR(hr,hr_exp) \
     ok(hr == hr_exp, "got 0x%08x, expected 0x%08x\n", hr, hr_exp)
 
+static const WCHAR winetestW[] = {'w','i','n','e','t','e','s','t',0};
+
 static HRESULT (WINAPI *pSHGetFolderPathW)(HWND, int, HANDLE, DWORD, LPWSTR);
 static HRESULT (WINAPI *pSHGetNameFromIDList)(PCIDLIST_ABSOLUTE,SIGDN,PWSTR*);
 static HRESULT (WINAPI *pSHGetSpecialFolderLocation)(HWND, int, LPITEMIDLIST *);
@@ -51,7 +53,6 @@ static void init_function_pointers(void)
 
 static void test_namespace(void)
 {
-    static const WCHAR winetestW[] = {'w','i','n','e','t','e','s','t',0};
     static const WCHAR backslashW[] = {'\\',0};
     static const WCHAR clsidW[] = {
         ':',':','{','6','4','5','F','F','0','4','0','-','5','0','8','1','-',
@@ -307,6 +308,136 @@ static void test_namespace(void)
     IShellDispatch_Release(sd);
 }
 
+static void test_items(void)
+{
+    WCHAR wstr[MAX_PATH], orig_dir[MAX_PATH];
+    HRESULT r;
+    IShellDispatch *sd = NULL;
+    Folder *folder = NULL;
+    FolderItems *items = NULL;
+    FolderItems2 *items2 = NULL;
+    FolderItems3 *items3 = NULL;
+    FolderItem *item = (FolderItem*)0xdeadbeef;
+    IDispatch *disp = NULL;
+    IUnknown *unk = NULL;
+    FolderItemVerbs *verbs = (FolderItemVerbs*)0xdeadbeef;
+    VARIANT var;
+    LONG lcount = -1;
+
+    r = CoCreateInstance(&CLSID_Shell, NULL, CLSCTX_INPROC_SERVER, &IID_IShellDispatch, (void**)&sd);
+    ok(SUCCEEDED(r), "CoCreateInstance failed: %08x\n", r);
+    ok(!!sd, "sd is null\n");
+
+    GetTempPathW(MAX_PATH, wstr);
+    GetCurrentDirectoryW(MAX_PATH, orig_dir);
+    SetCurrentDirectoryW(wstr);
+    CreateDirectoryW(winetestW, NULL);
+    GetFullPathNameW(winetestW, MAX_PATH, wstr, NULL);
+    V_VT(&var) = VT_BSTR;
+    V_BSTR(&var) = SysAllocString(wstr);
+    r = IShellDispatch_NameSpace(sd, var, &folder);
+    ok(r == S_OK, "IShellDispatch::NameSpace failed: %08x\n", r);
+    ok(!!folder, "folder is null\n");
+    SysFreeString(V_BSTR(&var));
+    IShellDispatch_Release(sd);
+    SetCurrentDirectoryW(winetestW);
+
+    r = Folder_Items(folder, &items);
+    ok(r == S_OK, "Folder::Items failed: %08x\n", r);
+    ok(!!items, "items is null\n");
+    r = FolderItems_QueryInterface(items, &IID_FolderItems2, (void**)&items2);
+    ok(r == S_OK || broken(E_NOINTERFACE) /* xp and later */, "FolderItems::QueryInterface failed: %08x\n", r);
+    ok(!!items2 || broken(!items2) /* xp and later */, "items2 is null\n");
+    r = FolderItems_QueryInterface(items, &IID_FolderItems3, (void**)&items3);
+    ok(r == S_OK, "FolderItems::QueryInterface failed: %08x\n", r);
+    ok(!!items3, "items3 is null\n");
+    Folder_Release(folder);
+
+    if (0) /* crashes on all versions of Windows */
+        r = FolderItems_get_Count(items, NULL);
+
+    r = FolderItems_get_Count(items, &lcount);
+todo_wine
+    ok(r == S_OK, "FolderItems::get_Count failed: %08x\n", r);
+todo_wine
+    ok(!lcount, "expected 0 files, got %d\n", lcount);
+
+    V_VT(&var) = VT_I4;
+    V_I4(&var) = 0;
+
+    if (0) /* crashes on all versions of Windows */
+        r = FolderItems_Item(items, var, NULL);
+
+    r = FolderItems_Item(items, var, &item);
+todo_wine
+    ok(r == S_FALSE, "expected S_FALSE, got %08x\n", r);
+    ok(!item, "item is not null\n");
+
+    if (0) /* crashes on xp */
+    {
+        r = FolderItems_get_Application(items, NULL);
+        ok(r == E_INVALIDARG, "expected E_INVALIDARG, got %08x\n", r);
+    }
+
+    r = FolderItems_get_Application(items, &disp);
+todo_wine
+    ok(r == S_OK, "FolderItems::get_Application failed: %08x\n", r);
+todo_wine
+    ok(!!disp, "disp is null\n");
+    if (disp) IDispatch_Release(disp);
+
+    if (0) /* crashes on xp */
+    {
+        r = FolderItems_get_Parent(items, NULL);
+        ok(r == E_NOTIMPL, "expected E_NOTIMPL, got %08x\n", r);
+    }
+
+    disp = (IDispatch*)0xdeadbeef;
+    r = FolderItems_get_Parent(items, &disp);
+    ok(r == E_NOTIMPL, "expected E_NOTIMPL, got %08x\n", r);
+    ok(!disp, "disp is not null\n");
+
+    if (0) /* crashes on xp */
+    {
+        r = FolderItems__NewEnum(items, NULL);
+        ok(r == E_INVALIDARG, "expected E_INVALIDARG, got %08x\n", r);
+    }
+
+    r = FolderItems__NewEnum(items, &unk);
+todo_wine
+    ok(r == S_OK, "FolderItems::_NewEnum failed: %08x\n", r);
+todo_wine
+    ok(!!unk, "unk is null\n");
+    if (unk) IUnknown_Release(unk);
+
+    if (items3)
+    {
+        r = FolderItems3_Filter(items3, 0, NULL);
+todo_wine
+        ok(r == S_OK, "expected S_OK, got %08x\n", r);
+
+        if (0) /* crashes on xp */
+        {
+            r = FolderItems3_get_Verbs(items3, NULL);
+            ok(r == E_INVALIDARG, "expected E_INVALIDARG, got %08x\n", r);
+        }
+
+        r = FolderItems3_get_Verbs(items3, &verbs);
+todo_wine
+        ok(r == S_FALSE, "expected S_FALSE, got %08x\n", r);
+        ok(!verbs, "verbs is not null\n");
+    }
+
+    GetTempPathW(MAX_PATH, wstr);
+    SetCurrentDirectoryW(wstr);
+    RemoveDirectoryW(winetestW);
+    SetCurrentDirectoryW(orig_dir);
+
+    FolderItems_Release(items);
+    if (items2) FolderItems2_Release(items2);
+    if (items3) FolderItems3_Release(items3);
+}
+
 static void test_service(void)
 {
     static const WCHAR spooler[] = {'S','p','o','o','l','e','r',0};
@@ -398,6 +529,7 @@ static void test_ShellFolderViewDual(void)
         &IID_IShellDispatch6,
         &IID_IShellDispatch5,
         &IID_IShellDispatch4,
+        &IID_IShellDispatch2,
         &IID_NULL
     };
     IShellFolderViewDual *viewdual;
@@ -450,7 +582,7 @@ if (0) /* crashes on pre-vista */ {
 
     disp = (void*)0xdeadbeef;
     hr = IShellView_GetItemObject(view, SVGIO_BACKGROUND, &IID_IShellFolderViewDual, (void**)&disp);
-    ok(hr == E_NOINTERFACE, "got 0x%08x\n", hr);
+    ok(hr == E_NOINTERFACE || broken(hr == E_NOTIMPL) /* win2k */, "got 0x%08x\n", hr);
     ok(disp == NULL, "got %p\n", disp);
     IShellView_Release(view);
 
@@ -703,14 +835,14 @@ static void test_ParseName(void)
 
     item = (void*)0xdeadbeef;
     hr = Folder_ParseName(folder, NULL, &item);
-    ok(hr == S_FALSE, "got 0x%08x\n", hr);
+    ok(hr == S_FALSE || broken(hr == E_INVALIDARG) /* win2k */, "got 0x%08x\n", hr);
     ok(item == NULL, "got %p\n", item);
 
     /* empty name */
     str = SysAllocStringLen(NULL, 0);
     item = (void*)0xdeadbeef;
     hr = Folder_ParseName(folder, str, &item);
-    ok(hr == S_FALSE, "got 0x%08x\n", hr);
+    ok(hr == S_FALSE || broken(hr == E_INVALIDARG) /* win2k */, "got 0x%08x\n", hr);
     ok(item == NULL, "got %p\n", item);
     SysFreeString(str);
 
@@ -718,7 +850,8 @@ static void test_ParseName(void)
     str = SysAllocString(cadabraW);
     item = (void*)0xdeadbeef;
     hr = Folder_ParseName(folder, str, &item);
-    ok(hr == S_FALSE, "got 0x%08x\n", hr);
+    ok(hr == S_FALSE || broken(hr == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND)) /* win2k */,
+        "got 0x%08x\n", hr);
     ok(item == NULL, "got %p\n", item);
     SysFreeString(str);
 
@@ -822,10 +955,51 @@ if (0) { /* crashes on winxp/win2k3 */
     ok(hr == S_OK, "got 0x%08x\n", hr);
     ok(verb == NULL, "got %p\n", verb);
 
+    FolderItemVerbs_Release(verbs);
     FolderItem_Release(item);
     IShellDispatch_Release(sd);
 }
 
+static void test_ShellExecute(void)
+{
+    HRESULT hr;
+    IShellDispatch2 *sd;
+    BSTR name;
+    VARIANT args, dir, op, show;
+
+    static const WCHAR regW[] = {'r','e','g',0};
+
+    hr = CoCreateInstance(&CLSID_Shell, NULL, CLSCTX_INPROC_SERVER,
+        &IID_IShellDispatch2, (void**)&sd);
+    if (hr != S_OK)
+    {
+        win_skip("IShellDispatch2 not supported\n");
+        return;
+    }
+
+    VariantInit(&args);
+    VariantInit(&dir);
+    VariantInit(&op);
+    VariantInit(&show);
+
+    V_VT(&show) = VT_I4;
+    V_I4(&show) = 0;
+
+    name = SysAllocString(regW);
+
+    hr = IShellDispatch2_ShellExecute(sd, name, args, dir, op, show);
+    ok(hr == S_OK, "ShellExecute failed: %08x\n", hr);
+
+    /* test invalid value for show */
+    V_VT(&show) = VT_BSTR;
+    V_BSTR(&show) = name;
+
+    hr = IShellDispatch2_ShellExecute(sd, name, args, dir, op, show);
+    ok(hr == S_OK, "ShellExecute failed: %08x\n", hr);
+
+    SysFreeString(name);
+}
+
 START_TEST(shelldispatch)
 {
     HRESULT r;
@@ -837,11 +1011,13 @@ START_TEST(shelldispatch)
 
     init_function_pointers();
     test_namespace();
+    test_items();
     test_service();
     test_ShellFolderViewDual();
     test_ShellWindows();
     test_ParseName();
     test_Verbs();
+    test_ShellExecute();
 
     CoUninitialize();
 }
