@@ -210,7 +210,7 @@ static BOOL buffer_process_converted_attribute(struct wined3d_buffer *buffer,
         const enum wined3d_buffer_conversion_type conversion_type,
         const struct wined3d_stream_info_element *attrib, DWORD *stride_this_run)
 {
-    DWORD attrib_size;
+    const struct wined3d_format *format = attrib->format;
     BOOL ret = FALSE;
     unsigned int i;
     DWORD_PTR data;
@@ -222,12 +222,12 @@ static BOOL buffer_process_converted_attribute(struct wined3d_buffer *buffer,
      */
     if (!attrib->stride)
     {
-        FIXME("%s used with stride 0, let's hope we get the vertex stride from somewhere else\n",
-                debug_d3dformat(attrib->format->id));
+        FIXME("%s used with stride 0, let's hope we get the vertex stride from somewhere else.\n",
+                debug_d3dformat(format->id));
     }
-    else if(attrib->stride != *stride_this_run && *stride_this_run)
+    else if (attrib->stride != *stride_this_run && *stride_this_run)
     {
-        FIXME("Got two concurrent strides, %d and %d\n", attrib->stride, *stride_this_run);
+        FIXME("Got two concurrent strides, %d and %d.\n", attrib->stride, *stride_this_run);
     }
     else
     {
@@ -237,7 +237,7 @@ static BOOL buffer_process_converted_attribute(struct wined3d_buffer *buffer,
             /* We rely that this happens only on the first converted attribute that is found,
              * if at all. See above check
              */
-            TRACE("Reconverting because converted attributes occur, and the stride changed\n");
+            TRACE("Reconverting because converted attributes occur, and the stride changed.\n");
             buffer->stride = *stride_this_run;
             HeapFree(GetProcessHeap(), HEAP_ZERO_MEMORY, buffer->conversion_map);
             buffer->conversion_map = wined3d_calloc(buffer->stride, sizeof(*buffer->conversion_map));
@@ -246,8 +246,7 @@ static BOOL buffer_process_converted_attribute(struct wined3d_buffer *buffer,
     }
 
     data = ((DWORD_PTR)attrib->data.addr) % buffer->stride;
-    attrib_size = attrib->format->component_count * attrib->format->component_size;
-    for (i = 0; i < attrib_size; ++i)
+    for (i = 0; i < format->attribute_size; ++i)
     {
         DWORD_PTR idx = (data + i) % buffer->stride;
         if (buffer->conversion_map[idx] != conversion_type)
@@ -382,6 +381,10 @@ static BOOL buffer_find_decl(struct wined3d_buffer *This, const struct wined3d_s
             fixup_flags, &stride_this_run) || ret;
     fixup_flags &= ~WINED3D_BUFFER_FIXUP_XYZRHW;
 
+    ret = buffer_check_attribute(This, si, state, WINED3D_FFP_BLENDWEIGHT,
+            fixup_flags, &stride_this_run) || ret;
+    ret = buffer_check_attribute(This, si, state, WINED3D_FFP_BLENDINDICES,
+            fixup_flags, &stride_this_run) || ret;
     ret = buffer_check_attribute(This, si, state, WINED3D_FFP_NORMAL,
             fixup_flags, &stride_this_run) || ret;
     ret = buffer_check_attribute(This, si, state, WINED3D_FFP_DIFFUSE,
@@ -556,28 +559,35 @@ static void wined3d_buffer_drop_bo(struct wined3d_buffer *buffer)
         device_invalidate_state(device, STATE_STREAMSRC);
 }
 
+static void wined3d_buffer_destroy_object(void *object)
+{
+    struct wined3d_buffer *buffer = object;
+    struct wined3d_context *context;
+
+    if (buffer->buffer_object)
+    {
+        context = context_acquire(buffer->resource.device, NULL);
+        delete_gl_buffer(buffer, context->gl_info);
+        context_release(context);
+
+        HeapFree(GetProcessHeap(), 0, buffer->conversion_map);
+    }
+
+    HeapFree(GetProcessHeap(), 0, buffer->maps);
+    HeapFree(GetProcessHeap(), 0, buffer);
+}
+
 ULONG CDECL wined3d_buffer_decref(struct wined3d_buffer *buffer)
 {
     ULONG refcount = InterlockedDecrement(&buffer->resource.ref);
-    struct wined3d_context *context;
 
     TRACE("%p decreasing refcount to %u.\n", buffer, refcount);
 
     if (!refcount)
     {
-        if (buffer->buffer_object)
-        {
-            context = context_acquire(buffer->resource.device, NULL);
-            delete_gl_buffer(buffer, context->gl_info);
-            context_release(context);
-
-            HeapFree(GetProcessHeap(), 0, buffer->conversion_map);
-        }
-
-        resource_cleanup(&buffer->resource);
         buffer->resource.parent_ops->wined3d_object_destroyed(buffer->resource.parent);
-        HeapFree(GetProcessHeap(), 0, buffer->maps);
-        HeapFree(GetProcessHeap(), 0, buffer);
+        resource_cleanup(&buffer->resource);
+        wined3d_cs_emit_destroy_object(buffer->resource.device->cs, wined3d_buffer_destroy_object, buffer);
     }
 
     return refcount;
@@ -1335,6 +1345,7 @@ static HRESULT buffer_init(struct wined3d_buffer *buffer, struct wined3d_device
         ERR("Out of memory.\n");
         buffer_unload(&buffer->resource);
         resource_cleanup(&buffer->resource);
+        wined3d_resource_wait_idle(&buffer->resource);
         return E_OUTOFMEMORY;
     }
     buffer->maps_size = 1;
@@ -1344,6 +1355,7 @@ static HRESULT buffer_init(struct wined3d_buffer *buffer, struct wined3d_device
         ERR("Failed to upload data, hr %#x.\n", hr);
         buffer_unload(&buffer->resource);
         resource_cleanup(&buffer->resource);
+        wined3d_resource_wait_idle(&buffer->resource);
         HeapFree(GetProcessHeap(), 0, buffer->maps);
         return hr;
     }
