@@ -53,6 +53,7 @@ enum wined3d_gl_extension
     ARB_DRAW_ELEMENTS_BASE_VERTEX,
     ARB_DRAW_INSTANCED,
     ARB_ES2_COMPATIBILITY,
+    ARB_EXPLICIT_ATTRIB_LOCATION,
     ARB_FRAGMENT_PROGRAM,
     ARB_FRAGMENT_SHADER,
     ARB_FRAMEBUFFER_OBJECT,
@@ -92,7 +93,9 @@ enum wined3d_gl_extension
     ARB_TEXTURE_RECTANGLE,
     ARB_TEXTURE_RG,
     ARB_TEXTURE_RGB10_A2UI,
+    ARB_TEXTURE_STORAGE,
     ARB_TEXTURE_SWIZZLE,
+    ARB_TEXTURE_VIEW,
     ARB_TIMER_QUERY,
     ARB_UNIFORM_BUFFER_OBJECT,
     ARB_VERTEX_ARRAY_BGRA,
@@ -100,6 +103,7 @@ enum wined3d_gl_extension
     ARB_VERTEX_BUFFER_OBJECT,
     ARB_VERTEX_PROGRAM,
     ARB_VERTEX_SHADER,
+    ARB_VERTEX_TYPE_2_10_10_10_REV,
     /* ATI */
     ATI_FRAGMENT_SHADER,
     ATI_SEPARATE_STENCIL,
@@ -128,6 +132,7 @@ enum wined3d_gl_extension
     EXT_STENCIL_TWO_SIDE,
     EXT_STENCIL_WRAP,
     EXT_TEXTURE3D,
+    EXT_TEXTURE_ARRAY,
     EXT_TEXTURE_COMPRESSION_RGTC,
     EXT_TEXTURE_COMPRESSION_S3TC,
     EXT_TEXTURE_ENV_COMBINE,
